@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23918 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2010-01-10 22:03:52 -0600 (Sun, 10 Jan 2010) $
  */
 
-#ifndef _ASNER_H
-#define _ASNER_H
+#ifndef PTLIB_ASNER_H
+#define PTLIB_ASNER_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -51,9 +51,6 @@ class PXMLElement;
 #undef      P_INCLUDE_XER
 #endif
 
-#ifndef _PTLIB_H
-#include <ptlib.h>
-#endif
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -601,6 +598,8 @@ class PASN_BMPString : public PASN_ConstrainedObject
     void SetValue(const PString & v) { operator=(v.AsUCS2()); }
     void SetValue(const PWCharArray & v) { operator=(v); }
     void SetValue(const PASN_BMPString & v) { operator=(v.value); }
+    void SetValueRaw(const PWCharArray & v) { SetValueRaw(v, v.GetSize()); }
+    void SetValueRaw(const wchar_t * val, PINDEX len);
 
     void SetCharacterSet(ConstraintType ctype, const char * charSet);
     void SetCharacterSet(ConstraintType ctype, const PWCharArray & charSet);
@@ -645,8 +644,8 @@ class PASN_GeneralisedTime : public PASN_VisibleString
       : PASN_VisibleString(UniversalGeneralisedTime, UniversalTagClass) { }
     PASN_GeneralisedTime(const PTime & time)
       : PASN_VisibleString(UniversalGeneralisedTime, UniversalTagClass) { SetValue(time); }
-    PASN_GeneralisedTime(unsigned tag, TagClass tagClass)
-      : PASN_VisibleString(tag, tagClass) { }
+    PASN_GeneralisedTime(unsigned theTag, TagClass theTagClass)
+      : PASN_VisibleString(theTag, theTagClass) { }
 
     PASN_GeneralisedTime & operator=(const PTime & time);
     void SetValue(const PTime & time) { operator=(time); }
@@ -662,8 +661,8 @@ class PASN_UniversalTime : public PASN_VisibleString
       : PASN_VisibleString(UniversalUTCTime, UniversalTagClass) { }
     PASN_UniversalTime(const PTime & time)
       : PASN_VisibleString(UniversalUTCTime, UniversalTagClass) { SetValue(time); }
-    PASN_UniversalTime(unsigned tag, TagClass tagClass)
-      : PASN_VisibleString(tag, tagClass) { }
+    PASN_UniversalTime(unsigned theTag, TagClass theTagClass)
+      : PASN_VisibleString(theTag, theTagClass) { }
 
     PASN_UniversalTime & operator=(const PTime & time);
     void SetValue(const PTime & time) { operator=(time); }
@@ -996,4 +995,7 @@ class PASN_Stream : public PBYTEArray
 #include "asnxer.h"
 #endif
 
-#endif // _ASNER_H
+#endif // PTLIB_ASNER_H
+
+
+// End Of File ///////////////////////////////////////////////////////////////
