@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 23959 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2010-01-21 03:18:08 -0600 (Thu, 21 Jan 2010) $
  */
 
 #ifndef PTLIB_HTTPFORM_H
@@ -241,7 +241,8 @@ class PHTTPCompositeField : public PHTTPField
     PHTTPCompositeField(
       const char * name,          // Name (identifier) for the field.
       const char * title = NULL,  // Title text for field (defaults to name).
-      const char * help = NULL    // Help text for the field.
+      const char * help = NULL,   // Help text for the field.
+      bool includeHeaders = false // Make a sub-table and put headers on HTML fields.
     );
 
     virtual void SetName(
@@ -308,6 +309,7 @@ class PHTTPCompositeField : public PHTTPField
 
   protected:
     PHTTPFields fields;
+    bool        m_includeHeaders;
 };
 
 
@@ -462,6 +464,32 @@ class PHTTPPasswordField : public PHTTPStringField
 };
 
 
+class PHTTPDateField : public PHTTPStringField
+{
+  PCLASSINFO(PHTTPDateField, PHTTPStringField)
+  public:
+    PHTTPDateField(
+      const char * name,
+      const PTime & initVal = PTime(0),
+      PTime::TimeFormat fmt = PTime::ShortDate
+    );
+
+    virtual PHTTPField * NewField() const;
+
+    virtual void SetValue(
+      const PString & newValue
+    );
+
+    virtual PBoolean Validated(
+      const PString & newValue,
+      PStringStream & msg
+    ) const;
+
+  protected:
+    PTime::TimeFormat m_format;
+};
+
+
 class PHTTPIntegerField : public PHTTPField
 {
   PCLASSINFO(PHTTPIntegerField, PHTTPField)
