@@ -27,9 +27,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22250 $
+ * $Revision: 23897 $
  * $Author: rjongbloed $
- * $Date: 2009-03-23 04:04:50 +0000 (Mon, 23 Mar 2009) $
+ * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
  */
 
 /**
@@ -46,7 +46,7 @@
    PODBCStmt      :  Wrapper for RecordSet (Internal)
    PODBCRecord    :  Handle Retrieve/Post/Bind Data from RecordSet (Internal)
 
-\begin{verbatim}
+<code>
   Example of Use
 
   PODBC link;
@@ -83,7 +83,7 @@
 // Disconnect from ODBC Source
   link.Disconnect();
 
-\end{verbatim}
+</code>
 */
 //--
 
