@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22754 $
+ * $Revision: 24065 $
  * $Author: rjongbloed $
- * $Date: 2009-05-28 12:49:36 +0000 (Thu, 28 May 2009) $
+ * $Date: 2010-02-15 18:01:44 -0600 (Mon, 15 Feb 2010) $
  */
 
 #ifndef PTLIB_CONTAIN_H
@@ -95,6 +95,10 @@
     #define _WIN32_WINNT WINVER
   #endif
 
+  #if !defined(_WIN32_WCE) && defined(_WIN32_WINNT) && (_WIN32_WINNT == 0x0500) && P_HAS_IPV6 && !defined(NTDDI_VERSION)
+    #define NTDDI_VERSION NTDDI_WIN2KSP1
+  #endif
+
   #ifndef STRICT
     #define STRICT
   #endif
