@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23897 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
  */
 
-#ifndef _PTCPSOCKET
-#define _PTCPSOCKET
+#ifndef PTLIB_TCPSOCKET_H
+#define PTLIB_TCPSOCKET_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -80,7 +80,7 @@ class PTCPSocket : public PIPSocket
   //@{
     /** Create a copy of the class on the heap. The exact semantics of the
        descendent class determine what is required to make a duplicate of the
-       instance. Not all classes can even {\bf do} a clone operation.
+       instance. Not all classes can even {\b do} a clone operation.
        
        The main user of the clone function is the #PDictionary# class as
        it requires copies of the dictionary keys.
@@ -123,6 +123,13 @@ class PTCPSocket : public PIPSocket
        of this type is then used to generate other "accepting" sockets which
        establish a two way communications channel with the "connecting" socket.
 
+       The Listen() method is only used once in the entire lifetime of
+       the socket. In contrast, the Accept() may be used many more
+       times. Thus, if you have a socket to collect incoming
+       connections, do a Listen() on it to get things ready, and then
+       Accept(). With each incoming connection, you will do Accept
+       again.
+
        If the #port# parameter is zero then the port number as
        defined by the object instance construction or the
        #PIPSocket::SetPort()# function.
@@ -212,6 +219,7 @@ class PTCPSocket : public PIPSocket
 #endif
 };
 
-#endif
+#endif // PTLIB_TCPSOCKET_H
+
 
 // End Of File ///////////////////////////////////////////////////////////////
