@@ -23,22 +23,19 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 20498 $
- * $Author: shorne $
- * $Date: 2008-06-26 01:05:26 +0000 (Thu, 26 Jun 2008) $
+ * $Revision: 23962 $
+ * $Author: rjongbloed $
+ * $Date: 2010-01-21 22:39:06 -0600 (Thu, 21 Jan 2010) $
  */
 
 
-#ifndef _PVIDEOIO
-#define _PVIDEOIO
+#ifndef PTLIB_PVIDEOIO_H
+#define PTLIB_PVIDEOIO_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
-
-#ifndef _PTLIB_H
-#include <ptlib.h>
-#endif
+#include <ptbuildopts.h>
 
 #if P_VIDEO
 
@@ -54,8 +51,6 @@ class PVideoFrameInfo : public PObject
   PCLASSINFO(PVideoFrameInfo, PObject);
 
   public:
-      PVideoFrameInfo();
-
     enum ResizeMode
     {
         eScale,
@@ -77,6 +72,16 @@ class PVideoFrameInfo : public PObject
       MaxWidth   = 1920, MaxHeight   = 1152
     };
 
+    /// Construct video frame information
+    PVideoFrameInfo();
+    PVideoFrameInfo(
+      unsigned        frameWidth,
+      unsigned        frameHeight,
+      const PString & colourFormat = "YUV420P",
+      unsigned        frameRate = 15,
+      ResizeMode      resizeMode = eScale
+    );
+
     /**Set the frame size to be used.
 
        Default behaviour sets the frameWidth and frameHeight variables and
@@ -158,14 +163,40 @@ class PVideoFrameInfo : public PObject
       const PString & colourFormat  ///< Colour format of frame
     );
 
-    /** Parse the standard size string names ("qcif", "cif" etc)
+    /** Parse a descriptor string for the video format.
+        This is of the form [fmt ':' ] size [ '@' rate][ '/' crop ]. The size component
+        is as for the ParseSize() function.
+
+        The fmt string is the colour format such as "RGB32", "YUV420P" etc.
+
+        The rate field is a simple integer from 1 to 100.
+
+        The crop field is one of "scale", "resize" (synonym for "scale"),
+        "centre", "center", "topleft" or "crop" (synonym for "topleft").
+
+        Note no spaces are allowed in the descriptor.
       */
-    static PBoolean ParseSize(
+    bool Parse(
+      const PString & str   ///< String to parse
+    );
+
+    /** Parse the standard size string names.
+        This will parse a size desciption using either standard names: "qcif",
+        "cif", "vga", "hd1080" etc or WxY form e.g. "640x480".
+      */
+    static bool ParseSize(
       const PString & str,  ///< String to parse
       unsigned & width,     ///< Resultant width
       unsigned & height     ///< Resulatant height
     );
 
+    /**Get a width/height as a standard size string name.
+      */
+    static PString AsString(
+      unsigned width,     ///< Width to convert
+      unsigned height     ///< Height to convert
+    );
+
   protected:
     unsigned   frameWidth;
     unsigned   frameHeight;
@@ -199,7 +230,7 @@ class PVideoControlInfo : public PObject
 };
 
 
-/**This class defines a video Input device control
+/**This class defines a video Input device control (Camera controls PTZ)
 */
 
 class PVideoInputControl : public PVideoControlInfo
@@ -232,6 +263,27 @@ protected:
 
 };
 
+/**This class defines a video Input device Interactions (Remote Inputs/Controls)
+*/
+class PVideoInteractionInfo : public PObject
+{
+  PCLASSINFO(PVideoInteractionInfo, PObject);
+
+ public:
+
+	 typedef enum {
+        InteractKey,		/// Register remote KeyPresses
+        InteractMouse,		/// Register remote Mouse Movements\Clicks
+        InteractNavigate,	/// Register remote Navigation commands
+		InteractRTSP,		/// Register remote RTSP (Real Time Streaming Protocol) Inputs
+		InteractOther		/// Register remote application specific Inputs
+     } InputInteractType;
+
+ 	static PString AsString(const InputInteractType & type);
+
+	InputInteractType type;
+};
+
 
 /**This class defines a video device.
    This class is used to abstract the few parameters that are common to both\
@@ -697,6 +749,17 @@ class PVideoOutputDevice : public PVideoDevice
       int & x,  // X position of device surface
       int & y   // Y position of device surface
     ) const;
+
+    /**Set the position of the output device, where relevant. For devices such as
+       files, this does nothing. For devices such as Windows, this sets the
+       position of the window on the screen.
+       
+       Returns: TRUE if the position can be set.
+      */
+    virtual bool SetPosition(
+      int x,  // X position of device surface
+      int y   // Y position of device surface
+    );
 };
 
 
@@ -885,10 +948,17 @@ class PVideoInputDevice : public PVideoDevice
 	typedef struct {
 	   std::list<PVideoFrameInfo> framesizes;
 	   std::list<PVideoControlInfo> controls;
+	   std::list<PVideoInteractionInfo> interactions;
 	} Capabilities;
 
     /**Retrieve a list of Device Capabilities
       */
+    virtual bool GetDeviceCapabilities(
+      Capabilities * capabilities          ///< List of supported capabilities
+    ) const { return GetDeviceCapabilities(GetDeviceName(), capabilities); }
+
+    /**Retrieve a list of Device Capabilities for particular device
+      */
     static PBoolean GetDeviceCapabilities(
       const PString & deviceName,           ///< Name of device
       Capabilities * capabilities,          ///< List of supported capabilities
@@ -976,6 +1046,28 @@ template <class className> class PVideoInputPluginServiceDescriptor : public PDe
   static PVideoInputPluginServiceDescriptor<PVideoInputDevice_##name> PVideoInputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoInputDevice, &PVideoInputDevice_##name##_descriptor)
 
+PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
+
+#ifdef P_APPSHARE
+  PPLUGIN_STATIC_LOAD(Application, PVideoInputDevice);
+#endif
+
+#if P_FFVDEV
+  PPLUGIN_STATIC_LOAD(FFMPEG, PVideoInputDevice);
+#endif
+
+#if P_VIDFILE
+  PPLUGIN_STATIC_LOAD(YUVFile, PVideoInputDevice);
+#endif
+
+#ifdef P_DIRECTSHOW
+  PPLUGIN_STATIC_LOAD(DirectShow, PVideoInputDevice);
+#endif
+
+#ifdef P_DIRECTSHOW2
+  PPLUGIN_STATIC_LOAD(DirectShow2, PVideoInputDevice);
+#endif
+
 ////////////////////////////////////////////////////////
 //
 // declare macros and structures needed for video output plugins
@@ -992,6 +1084,15 @@ template <class className> class PVideoOutputPluginServiceDescriptor : public PD
   static PVideoOutputPluginServiceDescriptor<PVideoOutputDevice_##name> PVideoOutputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoOutputDevice, &PVideoOutputDevice_##name##_descriptor)
 
+#if _WIN32
+  PPLUGIN_STATIC_LOAD(Window, PVideoOutputDevice);
+#endif
+
+#if P_SDL
+  PPLUGIN_STATIC_LOAD(SDL, PVideoOutputDevice);
+#endif
+
+
 ////////////////////////////////////////////////////////
 //
 // declare classes needed for access to simple video font
@@ -1014,6 +1115,6 @@ class PVideoFont : public PObject
 
 #endif // P_VIDEO
 
-#endif   // _PVIDEOIO
+#endif   // PTLIB_PVIDEOIO_H
 
 // End Of File ///////////////////////////////////////////////////////////////
