@@ -26,9 +26,9 @@
  *
  * Contributor(s): Loopback feature: Philip Edelbrock <phil@netroedge.com>.
  *
- * $Revision: 20949 $
- * $Author: ms30002000 $
- * $Date: 2008-09-14 09:02:41 +0000 (Sun, 14 Sep 2008) $
+ * $Revision: 23840 $
+ * $Author: csoutheren $
+ * $Date: 2009-12-04 16:41:11 -0600 (Fri, 04 Dec 2009) $
  */
 
 #pragma implementation "sound_oss.h"
@@ -556,8 +556,9 @@ PBoolean PSoundChannelOSS::Read(void * buf, PINDEX len)
         PTRACE(6, "OSS\tRead interrupted");
       }
       total += bytes;
-      if (total != len)
+      if (total != len) {
         PTRACE(6, "OSS\tRead completed short - " << total << " vs " << len << ". Reading more data");
+      }
     }
     lastReadCount = total;
   }
@@ -836,7 +837,7 @@ PBoolean PSoundChannelOSS::StartRecording()
   if (os_handle == 0)
     return PTrue;
 
-  P_fd_set fds = os_handle;
+  P_fd_set fds(os_handle);
   P_timeval instant;
   return ConvertOSError(::select(1, fds, NULL, NULL, instant));
 }
