@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23806 $
+ * $Revision: 24540 $
  * $Author: rjongbloed $
- * $Date: 2009-11-27 00:59:05 -0600 (Fri, 27 Nov 2009) $
+ * $Date: 2010-06-28 20:44:22 -0500 (Mon, 28 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -281,9 +281,12 @@ PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 
 PBoolean PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
 {
-  PBoolean ok = InternalReadContentBody(replyMIME, &body);
-  body.SetSize(body.GetSize()+1);
-  return ok;
+  PCharArray rawBody;
+  if (!InternalReadContentBody(replyMIME, &rawBody))
+    return false;
+
+  body = PString(rawBody, rawBody.GetSize());
+  return true;
 }
 
 
@@ -402,7 +405,7 @@ PBoolean PHTTPClient::GetTextDocument(const PURL & url,
 }
 
 
-bool PHTTPClient::GetDocument(const PURL & url,
+PBoolean PHTTPClient::GetDocument(const PURL & url,
                               PMIMEInfo & outMIME,
                               PMIMEInfo & replyMIME)
 {
@@ -648,8 +651,11 @@ PBoolean PHTTPClientBasicAuthentication::Authorise(AuthObject & authObject) cons
 ////////////////////////////////////////////////////////////////////////////////////
 
 PHTTPClientDigestAuthentication::PHTTPClientDigestAuthentication()
+  : algorithm(NumAlgorithms)
+  , qopAuth(false)
+  , qopAuthInt(false)
+  , stale(false)
 {
-  algorithm = NumAlgorithms;
 }
 
 PHTTPClientDigestAuthentication & PHTTPClientDigestAuthentication::operator =(const PHTTPClientDigestAuthentication & auth)
@@ -663,6 +669,7 @@ PHTTPClientDigestAuthentication & PHTTPClientDigestAuthentication::operator =(co
           
   qopAuth    = auth.qopAuth;
   qopAuthInt = auth.qopAuthInt;
+  stale      = auth.stale;
   cnonce     = auth.cnonce;
   nonceCount.SetValue(auth.nonceCount);
 
@@ -675,6 +682,9 @@ PObject::Comparison PHTTPClientDigestAuthentication::Compare(const PObject & oth
   if (otherAuth == NULL)
     return LessThan;
 
+  if (stale || otherAuth->stale)
+    return LessThan;
+
   if (algorithm < otherAuth->algorithm)
     return LessThan;
   if (algorithm > otherAuth->algorithm)
@@ -691,9 +701,9 @@ PObject::Comparison PHTTPClientDigestAuthentication::Compare(const PObject & oth
   return PHTTPClientAuthentication::Compare(other);
 }
 
-PBoolean PHTTPClientDigestAuthentication::Parse(const PString & _auth, PBoolean proxy)
+PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean proxy)
 {
-  PCaselessString auth =_auth;
+  PCaselessString auth = p_auth;
 
   authRealm.MakeEmpty();
   nonce.MakeEmpty();
@@ -747,6 +757,10 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & _auth, PBoolean
     cnonce = PGloballyUniqueID().AsString();
   }
 
+  PCaselessString staleStr = GetAuthParam(auth, "stale");
+  PTRACE_IF(3, !staleStr.IsEmpty(), "HTTP\tAuthentication contains stale flag \"" << staleStr << '"');
+  stale = staleStr.Find("true") != P_MAX_INDEX;
+
   isProxy = proxy;
   return PTrue;
 }
