@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 24367 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2010-05-21 22:15:14 -0500 (Fri, 21 May 2010) $
  */
 
 #include <ptlib.h>
@@ -40,7 +40,7 @@
 #define new PNEW
 
 
-// define to enable work-around for Netscape persistant connection bug
+// define to enable work-around for Netscape persistent connection bug
 // set to lifetime of suspect sockets (in seconds)
 #define STRANGE_NETSCAPE_BUG  3
 
@@ -229,186 +229,6 @@ void PHTTPServer::Construct()
   SetReadLineTimeout(PTimeInterval(0, READLINE_TIMEOUT));
 }
 
-void PHTTPConnectionInfo::DecodeMultipartFormInfo(const PString & type, const PString & entityBody)
-{
-  // remove trailing ","
-  PINDEX pos = type.Find(",");
-  if (pos == P_MAX_INDEX) {
-    pos = type.Find(";");
-    if (pos == P_MAX_INDEX) 
-      return;
-  }
-  PString seperator = type.Mid(pos+1).Trim();
-
-  // remove "boundary"
-  pos = seperator.Find("boundary");
-  if (pos == P_MAX_INDEX)
-    return;
-  seperator = seperator.Mid(8).Trim();
-
-  // remove "="
-  pos = seperator.Find("=");
-  if (pos == P_MAX_INDEX)
-    return;
-  seperator = seperator.Mid(1).Trim();
-
-  // seperators have a "--" according to RFC 1521
-  seperator = PString("--") + seperator;
-
-  PINDEX sepLen = seperator.GetLength();
-  const char * sep = (const char *)seperator;
-
-  // split body into parts, assuming binary data
-  const char * body = (const char *)entityBody;
-  PINDEX entityOffs = 0;
-  PINDEX entityLen = entityBody.GetSize()-1;
-
-  PBoolean ignore = PTrue;
-  PBoolean last = PFalse;
-
-  PMultipartFormInfo * info = NULL;
-
-  while (!last && (entityOffs < entityLen)) {
-
-    // find end of part
-    PINDEX partStart = entityOffs;
-    PINDEX partLen;
-    PBoolean foundSep = PFalse;
-
-    // collect length of part until seperator
-    for (partLen = 0; (partStart + partLen) < entityLen; partLen++) {
-      if ((partLen >= sepLen) && (memcmp(body + partStart + partLen - sepLen, sep, sepLen) == 0)) {
-        foundSep = PTrue;
-        break;
-      }
-    }
-
-    // move entity ptr to the end of the part
-    entityOffs = partStart + partLen;
-
-    // if no seperator found, then this is the last part
-    // otherwise, look for "--" trailer on seperator and remove CRLF
-    if (!foundSep)
-      last = PTrue;
-    else {
-      partLen -= sepLen;
-
-      // determine if this is the last block
-      if (((entityOffs + 2) <= entityLen) && (body[entityOffs] == '-') && (body[entityOffs+1] == '-')) {
-        last = PTrue;
-        entityOffs += 2;
-      }
-
-      // remove crlf
-      if (((entityOffs + 2) <= entityLen) && (body[entityOffs] == '\r') && (body[entityOffs+1] == '\n')) 
-        entityOffs += 2;
-    }
-
-    // ignore everything up to the first seperator, 
-    // then adjust seperator to include leading CRLF
-    if (ignore) {
-      ignore = PFalse;
-      seperator = PString("\r\n") + seperator;
-      sepLen = seperator.GetLength();
-      sep = (const char *)seperator;
-      continue;
-    }
-
-    // extract the MIME header, by looking for a double CRLF
-    PINDEX ptr;
-    PINDEX nlCount = 0;
-    for (ptr = partStart;(ptr < (partStart + partLen)) && (nlCount < 2); ptr++) {
-      if (body[ptr] == '\r') {
-        nlCount++;
-        if ((ptr < entityLen-1) && (body[ptr+1] == '\n'))
-          ptr++;
-      } else
-        nlCount = 0;
-    }
-
-    // create the new part info
-    info = new PMultipartFormInfo;
-
-    // read MIME information
-    PStringStream strm(PString(body + partStart, ptr - partStart));
-    info->mime.ReadFrom(strm);
-
-    // save the entity body, being careful of binary files
-    int savedLen = partStart + partLen - ptr;
-    char * saved = info->body.GetPointer(savedLen + 1);
-    memcpy(saved, body + ptr, savedLen);
-    saved[savedLen] = '\0';
-
-    // add the data to the array
-    multipartFormInfoArray.Append(info);
-    info = NULL;
-  }
-  
-#if 0
-  // ignore until first separator
-  do {
-    data >> line;
-    if (line.IsEmpty())
-      return;
-  } while (line.Find(sep) != 0);
-
-  PMultipartFormInfo * info = NULL;
-
-  // read form parts
-  while (data.good() && (line.Right(2) != "--")) {
-
-    info = new PMultipartFormInfo;
-
-    // read MIME information
-    info->mime.ReadFrom(data);
-
-    // get the content type
-    PString type = info->mime(PHTTP::ContentTypeTag);
-
-    // check the encoding
-    PString encoding = info->mime("Content-Transfer-Encoding");
-
-    // accumulate text until another seperator or end of data
-    PString & buf = info->body;
-    PINDEX len = 0;
-    buf.SetSize(len+1);
-    buf[0] = '\0';
-    PINDEX sepLen = sep.GetLength();
-    const char * sepPtr = (const char *)sep;
-    while (data.good()) {
-      buf.SetSize(len);
-      data >> buf[len++];
-      if ((len >= sepLen) && (memcmp(((const char *)buf) + len - sepLen, sepPtr, sepLen) == 0)) {
-        char ch;
-        data >> ch;
-        if (ch != 0x0d)
-          data.putback(ch); 
-        else {
-          data >> ch;
-          if (ch != 0x0a)
-            data.putback(ch); 
-        }
-        len -= sepLen;
-        break;
-      }
-    }
-    buf.SetSize(len+1);
-    buf[len] = '\0';
-
-    /*
-    while (data.good()) {
-      data >> line;
-      if (line.Find(sep) == 0)
-        break;
-      info->body += line + "\n";
-    } 
-    */
-
-    multipartFormInfoArray.Append(info);
-    info = NULL;
-  }
-#endif
-}
 
 PBoolean PHTTPServer::ProcessCommand()
 {
@@ -452,7 +272,7 @@ PBoolean PHTTPServer::ProcessCommand()
 
   // the URL that comes with Connect requests is not quite kosher, so 
   // mangle it into a proper URL and do NOT close the connection.
-  // for all other commands, close the read connection if not persistant
+  // for all other commands, close the read connection if not persistent
   if (cmd == CONNECT) 
     connectInfo.url = "https://" + args;
   else {
@@ -492,14 +312,8 @@ PBoolean PHTTPServer::ProcessCommand()
         break;
 
       case POST :
-        {
-          // check for multi-part form POSTs
-          PString postType = (connectInfo.GetMIME())(ContentTypeTag());
-          if (postType.Find("multipart/form-data") == 0)
-            connectInfo.DecodeMultipartFormInfo(postType, connectInfo.entityBody);
-          else  // if (postType *= "x-www-form-urlencoded)
-            PURL::SplitQueryVars(connectInfo.entityBody, postData);
-        }
+        if (!connectInfo.DecodeMultipartFormInfo())
+          PURL::SplitQueryVars(connectInfo.entityBody, postData); // x-www-form-urlencoded
         persist = OnPOST(url, connectInfo.GetMIME(), postData, connectInfo);
         break;
 
@@ -516,13 +330,13 @@ PBoolean PHTTPServer::ProcessCommand()
   // routines above must make sure that their return value is PFalse if
   // if there was no ContentLength field in the response. This ensures that
   // we always close the socket so the client will get the correct end of file
-  if (persist && connectInfo.IsPersistant()) {
+  if (persist && connectInfo.IsPersistent()) {
     unsigned max = connectInfo.GetPersistenceMaximumTransations();
     if (max == 0 || transactionCount < max)
       return PTrue;
   }
 
-  PTRACE(5, "HTTPServer\tConnection end: " << connectInfo.IsPersistant());
+  PTRACE(5, "HTTPServer\tConnection end: " << connectInfo.IsPersistent());
 
   // close the output stream now and return PFalse
   Shutdown(ShutdownWrite);
@@ -551,8 +365,8 @@ PString PHTTPServer::ReadEntityBody()
     entityBody.SetSize(count+1);
   }
 
-  // close the connection, if not persistant
-  if (!connectInfo.IsPersistant()) {
+  // close the connection, if not persistent
+  if (!connectInfo.IsPersistent()) {
     PIPSocket * socket = GetSocket();
     if (socket != NULL)
       socket->Shutdown(PIPSocket::ShutdownRead);
@@ -758,13 +572,13 @@ void PHTTPServer::SetDefaultMIMEInfo(PMIMEInfo & info,
   if (!info.Contains(ServerTag()))
     info.SetAt(ServerTag(), GetServerName());
 
-  if (connectInfo.IsPersistant()) {
+  if (connectInfo.IsPersistent()) {
     if (connectInfo.IsProxyConnection()) {
-      PTRACE(5, "HTTPServer\tSetting proxy persistant response");
+      PTRACE(5, "HTTPServer\tSetting proxy persistent response");
       info.SetAt(ProxyConnectionTag(), KeepAliveTag());
     }
     else {
-      PTRACE(5, "HTTPServer\tSetting direct persistant response");
+      PTRACE(5, "HTTPServer\tSetting direct persistent response");
       info.SetAt(ConnectionTag(), KeepAliveTag());
     }
   }
@@ -949,19 +763,20 @@ void PHTTPMultiSimpAuth::AddUser(const PString & username, const PString & passw
 
 PHTTPRequest::PHTTPRequest(const PURL & _url,
                       const PMIMEInfo & _mime,
-        const PMultipartFormInfoArray & _multipartFormInfo,
+                 const PMultiPartList & _multipartFormInfo,
+                        PHTTPResource * resource,
                           PHTTPServer & _server)
-  : server(_server),
-    url(_url),
-    inMIME(_mime),
-    multipartFormInfo(_multipartFormInfo),
-    origin(0),
-    localAddr(0),
-    localPort(0)
+  : server(_server)
+  , url(_url)
+  , inMIME(_mime)
+  , multipartFormInfo(_multipartFormInfo)
+  , code(PHTTP::RequestOK)
+  , contentSize(P_MAX_INDEX)
+  , origin(0)
+  , localAddr(0)
+  , localPort(0)
+  , m_resource(resource)
 {
-  code        = PHTTP::RequestOK;
-  contentSize = P_MAX_INDEX;
-
   PIPSocket * socket = server.GetSocket();
   if (socket != NULL) {
     socket->GetPeerAddress(origin);
@@ -974,9 +789,9 @@ PHTTPRequest::PHTTPRequest(const PURL & _url,
 // PHTTPConnectionInfo
 
 PHTTPConnectionInfo::PHTTPConnectionInfo()
-  : persistenceTimeout(0, DEFAULT_PERSIST_TIMEOUT) // maximum lifetime (in seconds) of persistant connections
+  : persistenceTimeout(0, DEFAULT_PERSIST_TIMEOUT) // maximum lifetime (in seconds) of persistent connections
 {
-  // maximum lifetime (in transactions) of persistant connections
+  // maximum lifetime (in transactions) of persistent connections
   persistenceMaximum = DEFAULT_PERSIST_TRANSATIONS;
 
   commandCode       = PHTTP::NumCommands;
@@ -984,13 +799,11 @@ PHTTPConnectionInfo::PHTTPConnectionInfo()
   majorVersion      = 0;
   minorVersion      = 9;
 
-  isPersistant      = PFalse;
-  wasPersistant     = PFalse;
-  isProxyConnection = PFalse;
+  isPersistent      = false;
+  wasPersistent     = false;
+  isProxyConnection = false;
 
   entityBodyLength  = -1;
-
-  multipartFormInfoArray.AllowDeleteObjects();
 }
 
 
@@ -1025,8 +838,8 @@ PBoolean PHTTPConnectionInfo::Initialise(PHTTPServer & server, PString & args)
   if (!mimeInfo.Read(server))
     return PFalse;
 
-  wasPersistant = isPersistant;
-  isPersistant = PFalse;
+  wasPersistent = isPersistent;
+  isPersistent = false;
 
   PString str;
 
@@ -1040,8 +853,8 @@ PBoolean PHTTPConnectionInfo::Initialise(PHTTPServer & server, PString & args)
   // get any connection options
   if (!str) {
     PStringArray tokens = str.Tokenise(", \r\n", PFalse);
-    for (PINDEX z = 0; !isPersistant && z < tokens.GetSize(); z++)
-      isPersistant = isPersistant || (tokens[z] *= PHTTP::KeepAliveTag());
+    for (PINDEX z = 0; !isPersistent && z < tokens.GetSize(); z++)
+      isPersistent = isPersistent || (tokens[z] *= PHTTP::KeepAliveTag());
   }
 
   // If the protocol is version 1.0 or greater, there is MIME info, and the
@@ -1049,20 +862,20 @@ PBoolean PHTTPConnectionInfo::Initialise(PHTTPServer & server, PString & args)
   // Content-Length header. If the protocol is less than version 1.0, then 
   // there is no entity body!
 
-  // if the client specified a persistant connection, then use the
+  // if the client specified a persistent connection, then use the
   // ContentLength field. If there is no content length field, then
   // assume a ContentLength of zero and close the connection.
   // The spec actually says to read until end of file in this case,
   // but Netscape hangs if this is done.
-  // If the client didn't specify a persistant connection, then use the
+  // If the client didn't specify a persistent connection, then use the
   // ContentLength if there is one or read until end of file if there isn't
-  if (!isPersistant)
+  if (!isPersistent)
     entityBodyLength = mimeInfo.GetInteger(PHTTP::ContentLengthTag(),
                                            (commandCode == PHTTP::POST) ? -2 : 0);
   else {
     entityBodyLength = mimeInfo.GetInteger(PHTTP::ContentLengthTag(), -1);
     if (entityBodyLength < 0) {
-      PTRACE(5, "HTTPServer\tPersistant connection has no content length");
+      PTRACE(5, "HTTPServer\tPersistent connection has no content length");
       entityBodyLength = 0;
       mimeInfo.SetAt(PHTTP::ContentLengthTag(), "0");
     }
@@ -1341,10 +1154,10 @@ PBoolean PHTTPResource::GetExpirationDate(PTime &)
 
 PHTTPRequest * PHTTPResource::CreateRequest(const PURL & url,
                                             const PMIMEInfo & inMIME,
-                                            const PMultipartFormInfoArray & multipartFormInfo,
+                                            const PMultiPartList & multipartFormInfo,
                                             PHTTPServer & socket)
 {
-  return new PHTTPRequest(url, inMIME, multipartFormInfo, socket);
+  return new PHTTPRequest(url, inMIME, multipartFormInfo, this, socket);
 }
 
 
@@ -1395,8 +1208,8 @@ PBoolean PHTTPResource::LoadData(PHTTPRequest & request, PCharArray & data)
 {
   PString text = LoadText(request);
   OnLoadedText(request, text);
-  text.SetSize(text.GetLength());  // Lose the trailing '\0'
-  data = text;
+  if(data.SetSize(text.GetLength()))
+    memcpy(data.GetPointer(), text.GetPointer(), text.GetLength()); // Lose the trailing '\0'
   return PFalse;
 }
 
@@ -1541,20 +1354,21 @@ PHTTPFile::PHTTPFile(const PURL & url,
 
 
 PHTTPFileRequest::PHTTPFileRequest(const PURL & url,
-                                   const PMIMEInfo & inMIME,
-                                   const PMultipartFormInfoArray & multipartFormInfo,
-                                   PHTTPServer & server)
-  : PHTTPRequest(url, inMIME, multipartFormInfo, server)
+                              const PMIMEInfo & inMIME,
+                         const PMultiPartList & multipartFormInfo,
+                                PHTTPResource * resource,
+                                  PHTTPServer & server)
+  : PHTTPRequest(url, inMIME, multipartFormInfo, resource, server)
 {
 }
 
 
 PHTTPRequest * PHTTPFile::CreateRequest(const PURL & url,
-                                        const PMIMEInfo & inMIME,
-                          const PMultipartFormInfoArray & multipartFormInfo,
-                PHTTPServer & server)
+                                   const PMIMEInfo & inMIME,
+                              const PMultiPartList & multipartFormInfo,
+                                       PHTTPServer & server)
 {
-  return new PHTTPFileRequest(url, inMIME, multipartFormInfo, server);
+  return new PHTTPFileRequest(url, inMIME, multipartFormInfo, this, server);
 }
 
 
@@ -1707,20 +1521,21 @@ PHTTPDirectory::PHTTPDirectory(const PURL & url,
 
 
 PHTTPDirRequest::PHTTPDirRequest(const PURL & url,
-                                 const PMIMEInfo & inMIME,
-                                 const PMultipartFormInfoArray & multipartFormInfo,
-                                 PHTTPServer & server)
-  : PHTTPFileRequest(url, inMIME, multipartFormInfo, server)
+                            const PMIMEInfo & inMIME,
+                       const PMultiPartList & multipartFormInfo,
+                              PHTTPResource * resource,
+                                PHTTPServer & server)
+  : PHTTPFileRequest(url, inMIME, multipartFormInfo, resource, server)
 {
 }
 
 
 PHTTPRequest * PHTTPDirectory::CreateRequest(const PURL & url,
                                         const PMIMEInfo & inMIME,
-                          const PMultipartFormInfoArray & multipartFormInfo,
-                          PHTTPServer & socket)
+                                   const PMultiPartList & multipartFormInfo,
+                                            PHTTPServer & socket)
 {
-  PHTTPDirRequest * request = new PHTTPDirRequest(url, inMIME, multipartFormInfo, socket);
+  PHTTPDirRequest * request = new PHTTPDirRequest(url, inMIME, multipartFormInfo, this, socket);
 
   const PStringArray & path = url.GetPath();
   request->realPath = basePath;
