@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23713 $
+ * $Revision: 23902 $
  * $Author: rjongbloed $
- * $Date: 2009-10-28 06:08:29 -0500 (Wed, 28 Oct 2009) $
+ * $Date: 2010-01-06 18:30:15 -0600 (Wed, 06 Jan 2010) $
  */
 
 #ifdef __GNUC__
@@ -52,12 +52,12 @@ PInternetProtocol::PInternetProtocol(const char * svcName,
                                      PINDEX cmdCount,
                                      char const * const * cmdNames)
   : defaultServiceName(svcName),
-    commandNames(cmdCount, cmdNames, PTrue),
+    commandNames(cmdCount, cmdNames, true),
     readLineTimeout(0, 10)   // 10 seconds
 {
   SetReadTimeout(PTimeInterval(0, 0, 10));  // 10 minutes
   stuffingState = DontStuff;
-  newLineToCRLF = PTrue;
+  newLineToCRLF = true;
   unReadCount = 0;
 }
 
@@ -112,11 +112,11 @@ PBoolean PInternetProtocol::Write(const void * buf, PINDEX len)
             if (newLineToCRLF) {
               if (current > base) {
                 if (!PIndirectChannel::Write(base, current - base))
-                  return PFalse;
+                  return false;
                 totalWritten += lastWriteCount;
               }
               if (!PIndirectChannel::Write("\r", 1))
-                return PFalse;
+                return false;
               totalWritten += lastWriteCount;
               base = current;
             }
@@ -131,11 +131,11 @@ PBoolean PInternetProtocol::Write(const void * buf, PINDEX len)
         if (*current == '.') {
           if (current > base) {
             if (!PIndirectChannel::Write(base, current - base))
-              return PFalse;
+              return false;
             totalWritten += lastWriteCount;
           }
           if (!PIndirectChannel::Write(".", 1))
-            return PFalse;
+            return false;
           totalWritten += lastWriteCount;
           base = current;
         }
@@ -150,7 +150,7 @@ PBoolean PInternetProtocol::Write(const void * buf, PINDEX len)
 
   if (current > base) {  
     if (!PIndirectChannel::Write(base, current - base))  
-      return PFalse;  
+      return false;  
     totalWritten += lastWriteCount;  
   }  
   
@@ -163,7 +163,7 @@ PBoolean PInternetProtocol::AttachSocket(PIPSocket * socket)
 {
   if (socket->IsOpen()) {
     if (Open(socket))
-      return PTrue;
+      return true;
     Close();
     SetErrorValues(Miscellaneous, 0x41000000);
   }
@@ -172,7 +172,7 @@ PBoolean PInternetProtocol::AttachSocket(PIPSocket * socket)
     delete socket;
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -239,9 +239,9 @@ PBoolean PInternetProtocol::WriteLine(const PString & line)
   PStringArray lines = line.Lines();
   for (PINDEX i = 0; i < lines.GetSize(); i++)
     if (!WriteString(lines[i] + CRLF))
-      return PFalse;
+      return false;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -251,11 +251,11 @@ PBoolean PInternetProtocol::ReadLine(PString & str, PBoolean allowContinuation)
 
   PCharArray line(100);
   PINDEX count = 0;
-  PBoolean gotEndOfLine = PFalse;
+  PBoolean gotEndOfLine = false;
 
   int c = ReadChar();
   if (c < 0)
-    return PFalse;
+    return false;
 
   PTimeInterval oldTimeout = GetReadTimeout();
   SetReadTimeout(readLineTimeout);
@@ -298,10 +298,10 @@ PBoolean PInternetProtocol::ReadLine(PString & str, PBoolean allowContinuation)
 
       case '\n' :
         if (count == 0 || !allowContinuation || (c = ReadChar()) < 0)
-          gotEndOfLine = PTrue;
+          gotEndOfLine = true;
         else if (c != ' ' && c != '\t') {
           UnRead(c);
-          gotEndOfLine = PTrue;
+          gotEndOfLine = true;
         }
         break;
 
@@ -348,7 +348,7 @@ void PInternetProtocol::UnRead(const void * buffer, PINDEX len)
 PBoolean PInternetProtocol::WriteCommand(PINDEX cmdNumber)
 {
   if (cmdNumber >= commandNames.GetSize())
-    return PFalse;
+    return false;
   return WriteLine(commandNames[cmdNumber]);
 }
 
@@ -356,7 +356,7 @@ PBoolean PInternetProtocol::WriteCommand(PINDEX cmdNumber)
 PBoolean PInternetProtocol::WriteCommand(PINDEX cmdNumber, const PString & param)
 {
   if (cmdNumber >= commandNames.GetSize())
-    return PFalse;
+    return false;
   if (param.IsEmpty())
     return WriteLine(commandNames[cmdNumber]);
   else
@@ -368,7 +368,7 @@ PBoolean PInternetProtocol::ReadCommand(PINDEX & num, PString & args)
 {
   do {
     if (!ReadLine(args))
-      return PFalse;
+      return false;
   } while (args.IsEmpty());
 
   PINDEX endCommand = args.Find(' ');
@@ -380,7 +380,7 @@ PBoolean PInternetProtocol::ReadCommand(PINDEX & num, PString & args)
   if (num != P_MAX_INDEX)
     args = args.Mid(endCommand+1);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -400,7 +400,7 @@ PBoolean PInternetProtocol::WriteResponse(const PString & code,
   PINDEX i;
   for (i = 0; i < lines.GetSize()-1; i++)
     if (!WriteString(code + '-' + lines[i] + CRLF))
-      return PFalse;
+      return false;
 
   return WriteString((code & lines[i]) + CRLF);
 }
@@ -417,12 +417,12 @@ PBoolean PInternetProtocol::ReadResponse()
       lastResponseInfo = "Remote shutdown";
       SetErrorValues(ProtocolFailure, 0, LastReadError);
     }
-    return PFalse;
+    return false;
   }
 
   PINDEX continuePos = ParseResponse(line);
   if (continuePos == 0)
-    return PTrue;
+    return true;
 
   PString prefix = line.Left(continuePos);
   char continueChar = line[continuePos];
@@ -434,7 +434,7 @@ PBoolean PInternetProtocol::ReadResponse()
         lastResponseInfo += GetErrorText(LastReadError);
       else
         SetErrorValues(ProtocolFailure, 0, LastReadError);
-      return PFalse;
+      return false;
     }
     if (line.Left(continuePos) == prefix)
       lastResponseInfo += line.Mid(continuePos+1);
@@ -442,7 +442,7 @@ PBoolean PInternetProtocol::ReadResponse()
       lastResponseInfo += line;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -518,6 +518,14 @@ PMIMEInfo::PMIMEInfo(PInternetProtocol & socket)
 
 
 void PMIMEInfo::PrintOn(ostream &strm) const
+{
+  PrintContents(strm);
+  if (strm.fill() == '\r')
+    strm << '\r';
+  strm << '\n';
+}
+
+ostream & PMIMEInfo::PrintContents(ostream &strm) const
 {
   PBoolean output_cr = strm.fill() == '\r';
   strm.fill(' ');
@@ -540,9 +548,7 @@ void PMIMEInfo::PrintOn(ostream &strm) const
       strm << '\n';
     }
   }
-  if (output_cr)
-    strm << '\r';
-  strm << '\n';
+  return strm;
 }
 
 
@@ -575,21 +581,21 @@ PBoolean PMIMEInfo::Read(PInternetProtocol & socket)
   RemoveAll();
 
   PString line;
-  while (socket.ReadLine(line, PTrue)) {
+  while (socket.ReadLine(line, true)) {
     if (line.IsEmpty())
-      return PTrue;
+      return true;
     AddMIME(line);
   }
 
-  return PFalse;
+  return false;
 }
 
 
-PBoolean PMIMEInfo::AddMIME(const PString & line)
+bool PMIMEInfo::AddMIME(const PString & line)
 {
   PINDEX colonPos = line.Find(':');
   if (colonPos == P_MAX_INDEX)
-    return PFalse;
+    return false;
 
   PCaselessString fieldName  = line.Left(colonPos).Trim();
   PString fieldValue = line(colonPos+1, P_MAX_INDEX).Trim();
@@ -597,16 +603,27 @@ PBoolean PMIMEInfo::AddMIME(const PString & line)
   return AddMIME(fieldName, fieldValue);
 }
 
-PBoolean PMIMEInfo::AddMIME(const PString & fieldName, const PString & _fieldValue)
+
+bool PMIMEInfo::AddMIME(const PString & fieldName, const PString & fieldValue)
 {
-  PString fieldValue(_fieldValue);
+  PString * str = GetAt(fieldName);
+  if (str == NULL)
+    return SetAt(fieldName, fieldValue);
 
-  if (Contains(fieldName))
-    fieldValue = (*this)[fieldName] + '\n' + fieldValue;
+  *str += '\n';
+  *str += fieldValue;
+  return true;
+}
 
-  SetAt(fieldName, fieldValue);
 
-  return PTrue;
+bool PMIMEInfo::AddMIME(const PMIMEInfo & mime)
+{
+  for (PINDEX i = 0; i < mime.GetSize(); ++i) {
+    if (!AddMIME(mime.GetKeyAt(i), mime.GetDataAt(i)))
+      return false;
+  }
+
+  return true;
 }
 
 
@@ -619,12 +636,12 @@ PBoolean PMIMEInfo::Write(PInternetProtocol & socket) const
       PStringArray vals = value.Lines();
       for (PINDEX j = 0; j < vals.GetSize(); j++) {
         if (!socket.WriteLine(name + vals[j]))
-          return PFalse;
+          return false;
       }
     }
     else {
       if (!socket.WriteLine(name + value))
-        return PFalse;
+        return false;
     }
   }
 
@@ -634,17 +651,15 @@ PBoolean PMIMEInfo::Write(PInternetProtocol & socket) const
 
 PString PMIMEInfo::GetString(const PString & key, const PString & dflt) const
 {
-  if (GetAt(PCaselessString(key)) == NULL)
-    return dflt;
-  return operator[](key);
+  PString * str = GetAt(PCaselessString(key));
+  return str != NULL ? *str : dflt;
 }
 
 
 long PMIMEInfo::GetInteger(const PString & key, long dflt) const
 {
-  if (GetAt(PCaselessString(key)) == NULL)
-    return dflt;
-  return operator[](key).AsInteger();
+  PString * str = GetAt(PCaselessString(key));
+  return str != NULL ? str->AsInteger() : dflt;
 }
 
 
@@ -654,6 +669,97 @@ void PMIMEInfo::SetInteger(const PCaselessString & key, long value)
 }
 
 
+bool PMIMEInfo::ParseComplex(const PString & fieldValue, PStringToString & info)
+{
+  info.RemoveAll();
+
+  PStringArray fields = fieldValue.Lines();
+  for (PINDEX f = 0; f < fields.GetSize(); ++f) {
+    PString field = fields[f];
+
+    PINDEX tagSep = 0;
+    while (tagSep != P_MAX_INDEX) {
+      if (field[tagSep] == ',') {
+        while (isspace(field[tagSep]) || field[tagSep] == ',')
+          tagSep++;
+        if (field[tagSep] == '\0')
+          break;
+      }
+
+      if (field[tagSep] == ';')
+        continue;
+
+      PString keyPrefix;
+
+      if (!info.IsEmpty()) {
+        unsigned count = 0;
+        do {
+          keyPrefix = psprintf("%u:", ++count);
+        } while (info.Contains(keyPrefix));
+      }
+
+      if (field[tagSep] != '<') {
+        PINDEX nextSep = field.FindOneOf(";,", tagSep);
+        info.SetAt(keyPrefix, field(tagSep, nextSep-1).Trim());
+        tagSep = nextSep;
+      }
+      else {
+        PINDEX endtoken = field.Find('>', tagSep);
+        info.SetAt(keyPrefix, field(tagSep+1, endtoken-1));
+        tagSep = field.FindOneOf(";,", endtoken);
+      }
+
+      while (tagSep != P_MAX_INDEX && field[tagSep] != ',') {
+        ++tagSep; // Skip past ';'
+        PINDEX pos = field.FindOneOf("=;,", tagSep);
+        PCaselessString tag = field(tagSep, pos-1).Trim();
+
+        if (pos == P_MAX_INDEX || field[pos] == ',') {
+          info.SetAt(keyPrefix+tag, PString::Empty());
+          break;
+        }
+
+        if (field[pos] == ';') {
+          info.SetAt(keyPrefix+tag, PString::Empty());
+          tagSep = pos;
+          continue;
+        }
+
+        do {
+          ++pos;
+        } while (isspace(field[pos]));
+
+        if (field[pos] != '"') {
+          tagSep = field.FindOneOf(";,", pos);
+          info.SetAt(keyPrefix+tag, PCaselessString(field(pos, tagSep-1).RightTrim()));
+          continue;
+        }
+
+        ++pos;
+        PINDEX quote = pos;
+        while ((quote = field.Find('"', quote)) != P_MAX_INDEX && field[quote-1] == '\\')
+          ++quote;
+
+        PString value = field(pos, quote-1);
+        value.Replace("\\", "", true);
+        info.SetAt(keyPrefix+tag, value);
+
+        tagSep = field.FindOneOf(";,", quote);
+      }
+    }
+  }
+
+  return !info.IsEmpty();
+}
+
+
+bool PMIMEInfo::DecodeMultiPartList(PMultiPartList & parts, const PString & body, const PString & key) const
+{
+  PStringToString info;
+  return GetComplex(key, info) && parts.Decode(body, info);
+}
+
+
 static const PStringToString::Initialiser DefaultContentTypes[] = {
   { ".txt", "text/plain" },
   { ".text", "text/plain" },
@@ -682,7 +788,7 @@ PStringToString & PMIMEInfo::GetContentTypes()
 {
   static PStringToString contentTypes(PARRAYSIZE(DefaultContentTypes),
                                       DefaultContentTypes,
-                                      PTrue);
+                                      true);
   return contentTypes;
 }
 
@@ -709,4 +815,137 @@ PString PMIMEInfo::GetContentType(const PString & fType)
   return "application/octet-stream";
 }
 
+
+const PString & PMIMEInfo::ContentTypeTag()             { static PString s = "Content-Type";              return s; }
+const PString & PMIMEInfo::ContentDispositionTag()      { static PString s = "Content-Disposition";       return s; }
+const PString & PMIMEInfo::ContentTransferEncodingTag() { static PString s = "Content-Transfer-Encoding"; return s; }
+const PString & PMIMEInfo::ContentDescriptionTag()      { static PString s = "Content-Description";       return s; }
+const PString & PMIMEInfo::ContentIdTag()               { static PString s = "Content-ID";                return s; }
+
+
+//////////////////////////////////////////////////////////////////////////////
+
+static PINDEX FindBoundary(const PString & boundary, const char * & bodyPtr, PINDEX & bodyLen)
+{
+  PINDEX boundaryLen = boundary.GetLength();
+  const char * base = bodyPtr;
+  const char * found;
+  while (bodyLen >= boundaryLen && (found = (const char *)memchr(bodyPtr, boundary[0], bodyLen)) != NULL) {
+    PINDEX skip = found - bodyPtr + 1;
+    bodyPtr += skip;
+    bodyLen -= skip;
+
+    if (bodyLen >= boundaryLen && memcmp(found, (const char *)boundary, boundaryLen) == 0) {
+      bodyPtr += boundaryLen;
+      bodyLen -= boundaryLen;
+      if (bodyLen < 2)
+        return P_MAX_INDEX;
+
+      if (bodyPtr[0] == '\r') {
+        ++bodyPtr;
+        --bodyLen;
+      }
+      if (bodyPtr[0] == '\n') {
+        ++bodyPtr;
+        --bodyLen;
+      }
+
+      PINDEX len = found - base;
+      if (len > 0 && *(found-1) == '\n') {
+        --len;
+        if (len > 0 && *(found-2) == '\r')
+          --len;
+      }
+      return len;
+    }
+  }
+
+  return P_MAX_INDEX;
+}
+
+bool PMultiPartList::Decode(const PString & entityBody, const PStringToString & contentInfo)
+{
+  RemoveAll();
+
+  if (entityBody.IsEmpty())
+    return false;
+
+  PCaselessString multipartContentType = contentInfo(PString::Empty());
+  if (multipartContentType.NumCompare("multipart/") != EqualTo)
+    return false;
+
+  if (!contentInfo.Contains("boundary")) {
+    PTRACE(2, "MIME\tNo boundary in multipart Content-Type");
+    return false;
+  }
+
+  PCaselessString startContentId, startContentType;
+  if (multipartContentType == "multipart/related") {
+    startContentId = contentInfo("start");
+    startContentType = contentInfo("type");
+  }
+
+  PString boundary = "--" + contentInfo["boundary"];
+
+  // split body into parts, assuming binary data
+  const char * bodyPtr = (const char *)entityBody;
+  PINDEX bodyLen = entityBody.GetSize()-1;
+
+  // Find first boundary
+  if (FindBoundary(boundary, bodyPtr, bodyLen) == P_MAX_INDEX) {
+    PTRACE(2, "MIME\tNo boundary found in multipart body");
+    return false;
+  }
+
+  for (;;) {
+    const char * partPtr = bodyPtr;
+    PINDEX partLen = FindBoundary(boundary, bodyPtr, bodyLen);
+    if (partLen == P_MAX_INDEX)
+      break;
+
+    // create the new part info
+    PMultiPartInfo * info = new PMultiPartInfo;
+
+    // read MIME information
+    PStringStream strm(PString(partPtr, partLen));
+    info->m_mime.ReadFrom(strm);
+
+    // Skip over MIME
+    partPtr += strm.tellp();
+    partLen -= strm.tellp();
+
+    // Check transfer encoding
+    PStringToString typeInfo;
+    info->m_mime.GetComplex(PMIMEInfo::ContentTypeTag, typeInfo);
+    PCaselessString encoding = info->m_mime.GetString(PMIMEInfo::ContentTransferEncodingTag);
+
+    // save the entity body, being careful of binary files
+    if (encoding == "base64")
+      PBase64::Decode(PString(partPtr, partLen), info->m_binaryBody);
+    else if (typeInfo("charset") *= "UCS-2")
+      info->m_textBody = PString((const wchar_t *)partPtr, partLen/2);
+    else if (encoding == "7bit" || encoding == "8bit" || (typeInfo("charset") *= "UTF-8") || memchr(partPtr, 0, partLen) == NULL)
+      info->m_textBody = PString(partPtr, partLen);
+    else
+      info->m_binaryBody = PBYTEArray((const BYTE *)partPtr, partLen);
+
+    // add the data to the array
+    if (startContentId.IsEmpty() || startContentId != info->m_mime.GetString(PMIMEInfo::ContentIdTag))
+      Append(info);
+    else {
+      // Make sure start Content Type is set
+      if (!info->m_mime.Contains(PMIMEInfo::ContentTypeTag))
+        info->m_mime.SetAt(PMIMEInfo::ContentTypeTag, startContentType);
+
+      // Make sure "start" mime entry is at the beginning of list
+      InsertAt(0, info);
+      startContentId.MakeEmpty();
+    }
+
+  }
+
+  return !IsEmpty();
+}
+
+
 // End Of File ///////////////////////////////////////////////////////////////
