@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21695 $
+ * $Revision: 24410 $
  * $Author: rjongbloed $
- * $Date: 2008-12-04 02:01:41 +0000 (Thu, 04 Dec 2008) $
+ * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
  */
 
 //////////////////////////////////////////////////
@@ -42,8 +42,11 @@
 static const char FactoryName[] = PINTERFACE_MONITOR_FACTORY_NAME;
 static PFactory<PProcessStartup>::Worker<PInterfaceMonitor> InterfaceMonitorFactory(FactoryName, true);
 
-static int UDP_BUFFER_SIZE = 32767;
-
+#ifndef _WIN32_WCE
+#define UDP_BUFFER_SIZE 32768
+#else
+#define UDP_BUFFER_SIZE 8192
+#endif
 
 #define new PNEW
 
@@ -51,7 +54,7 @@ static int UDP_BUFFER_SIZE = 32767;
 //////////////////////////////////////////////////
 
 PInterfaceMonitorClient::PInterfaceMonitorClient(PINDEX _priority)
-: priority(_priority)
+  : priority(_priority)
 {
   PInterfaceMonitor::GetInstance().AddClient(this);
 }
@@ -69,7 +72,7 @@ PStringArray PInterfaceMonitorClient::GetInterfaces(bool includeLoopBack, const
 }
 
 
-PBoolean PInterfaceMonitorClient::GetInterfaceInfo(const PString & iface, InterfaceEntry & info)
+PBoolean PInterfaceMonitorClient::GetInterfaceInfo(const PString & iface, InterfaceEntry & info) const
 {
   return PInterfaceMonitor::GetInstance().GetInterfaceInfo(iface, info);
 }
@@ -81,8 +84,8 @@ PInterfaceMonitor::PInterfaceMonitor(unsigned refresh, bool runMonitorThread)
   : m_runMonitorThread(runMonitorThread)
   , m_refreshInterval(refresh)
   , m_updateThread(NULL)
-  , m_threadRunning(false)
   , m_interfaceFilter(NULL)
+  , m_changedDetector(NULL)
 {
 }
 
@@ -91,6 +94,7 @@ PInterfaceMonitor::~PInterfaceMonitor()
 {
   Stop();
 
+  delete m_changedDetector;
   delete m_interfaceFilter;
 }
 
@@ -116,17 +120,15 @@ void PInterfaceMonitor::SetRunMonitorThread(bool runMonitorThread)
 void PInterfaceMonitor::Start()
 {
   PWaitAndSignal guard(m_threadMutex);
-  
-  if (m_updateThread != NULL) // Already running
-    m_signalUpdate.Signal();
-  else {
+
+  if (m_changedDetector == NULL) {
     m_interfacesMutex.Wait();
     PIPSocket::GetInterfaceTable(m_interfaces);
-    PTRACE(4, "IfaceMon\tInitial interface list:\n" << setfill('\n') << m_interfaces << setfill(' '));
+    PTRACE(3, "IfaceMon\tInitial interface list:\n" << setfill('\n') << m_interfaces << setfill(' '));
     m_interfacesMutex.Signal();
 
     if (m_runMonitorThread) {
-      m_threadRunning = true;
+      m_changedDetector = PIPSocket::CreateRouteTableDetector();
       m_updateThread = new PThreadObj<PInterfaceMonitor>(*this, &PInterfaceMonitor::UpdateThreadMain);
       m_updateThread->SetThreadName("Network Interface Monitor");
     }
@@ -139,9 +141,10 @@ void PInterfaceMonitor::Stop()
   m_threadMutex.Wait();
 
   // shutdown the update thread
-  if (m_updateThread != NULL) {
-    m_threadRunning = false;
-    m_signalUpdate.Signal();
+  if (m_changedDetector != NULL) {
+    PTRACE(4, "IfaceMon\tAwaiting thread termination");
+
+    m_changedDetector->Cancel();
 
     m_threadMutex.Signal();
     m_updateThread->WaitForTermination();
@@ -149,6 +152,9 @@ void PInterfaceMonitor::Stop()
 
     delete m_updateThread;
     m_updateThread = NULL;
+
+    delete m_changedDetector;
+    m_changedDetector = NULL;
   }
 
   m_threadMutex.Signal();
@@ -219,7 +225,7 @@ void PInterfaceMonitor::RefreshInterfaceList()
   PIPSocket::InterfaceTable oldInterfaces = m_interfaces;
   m_interfaces = newInterfaces;
 
-  PTRACE(4, "IfaceMon\tInterface change detected, new list:\n" << setfill('\n') << newInterfaces << setfill(' '));
+  PTRACE(3, "IfaceMon\tInterface change detected, new list:\n" << setfill('\n') << newInterfaces << setfill(' '));
   
   m_interfacesMutex.Signal();
 
@@ -253,10 +259,8 @@ void PInterfaceMonitor::UpdateThreadMain()
   PTRACE(4, "IfaceMon\tStarted interface monitor thread.");
 
   // check for interface changes periodically
-  while (m_threadRunning) {
+  while (m_changedDetector->Wait(m_refreshInterval))
     RefreshInterfaceList();
-    PIPSocket::WaitForRouteTableChange(m_refreshInterval, &m_signalUpdate);
-  }
 
   PTRACE(4, "IfaceMon\tFinished interface monitor thread.");
 }
@@ -264,7 +268,7 @@ void PInterfaceMonitor::UpdateThreadMain()
 
 static PString MakeInterfaceDescription(const PIPSocket::InterfaceEntry & entry)
 {
-  return entry.GetAddress().AsString() + '%' + entry.GetName();
+  return entry.GetAddress().AsString(PTrue) + '%' + entry.GetName();
 }
 
 
@@ -354,7 +358,7 @@ bool PInterfaceMonitor::IsValidBindingForDestination(const PIPSocket::Address &
 }
 
 
-bool PInterfaceMonitor::GetInterfaceInfo(const PString & iface, PIPSocket::InterfaceEntry & info)
+bool PInterfaceMonitor::GetInterfaceInfo(const PString & iface, PIPSocket::InterfaceEntry & info) const
 {
   PIPSocket::Address addr;
   PString name;
@@ -465,6 +469,7 @@ PMonitoredSockets::PMonitoredSockets(bool reuseAddr, PNatMethod  * nat)
   , reuseAddress(reuseAddr)
   , natMethod(nat)
   , opened(false)
+  , interfaceAddedSignal(localPort, PIPSocket::GetDefaultIpAddressFamily())
 {
 }
 
@@ -488,7 +493,7 @@ bool PMonitoredSockets::CreateSocket(SocketInfo & info, const PIPSocket::Address
     }
   }
 
-  info.socket = new PUDPSocket;
+  info.socket = new PUDPSocket(localPort, (int) (binding.GetVersion() == 6 ? AF_INET6 : AF_INET));
   if (info.socket->Listen(binding, 0, localPort, reuseAddress?PIPSocket::CanReuseAddress:PIPSocket::AddressIsExclusive)) {
     PTRACE(4, "MonSock\tCreated bundled UDP socket " << binding << ':' << info.socket->GetPort());
     int sz = 0;
@@ -877,7 +882,8 @@ PBoolean PMonitoredSocketBundle::GetAddress(const PString & iface,
                                         WORD & port,
                                         PBoolean usingNAT) const
 {
-  if (iface.IsEmpty()) {
+  PIPSocket::InterfaceEntry info;
+  if (!GetInterfaceInfo(iface, info)) {
     address = PIPSocket::GetDefaultIpAny();
     port = localPort;
     return true;
@@ -887,7 +893,7 @@ PBoolean PMonitoredSocketBundle::GetAddress(const PString & iface,
   if (!guard.IsLocked())
     return false;
 
-  SocketInfoMap_T::const_iterator iter = socketInfoMap.find(iface);
+  SocketInfoMap_T::const_iterator iter = socketInfoMap.find(MakeInterfaceDescription(info));
   return iter != socketInfoMap.end() && GetSocketAddress(iter->second, address, port, usingNAT);
 }
 
@@ -1048,7 +1054,7 @@ PSingleMonitoredSocket::~PSingleMonitoredSocket()
 }
 
 
-PStringArray PSingleMonitoredSocket::GetInterfaces(PBoolean /*includeLoopBack*/, const PIPSocket::Address & /*destination*/)
+PStringArray PSingleMonitoredSocket::GetInterfaces(bool /*includeLoopBack*/, const PIPSocket::Address & /*destination*/)
 {
   PSafeLockReadOnly guard(*this);
 
@@ -1074,15 +1080,15 @@ PBoolean PSingleMonitoredSocket::Open(WORD port)
 
   if (theEntry.GetAddress().IsAny())
     GetInterfaceInfo(theInterface, theEntry);
-
+    
   if (theEntry.GetAddress().IsAny()) {
     PTRACE(3, "MonSock\tNot creating socket as interface \"" << theEntry.GetName() << "\" is  not up.");
     return true; // Still say successful though
   }
-
+    
   if (!CreateSocket(theInfo, theEntry.GetAddress()))
     return false;
-
+    
   localPort = theInfo.socket->PUDPSocket::GetPort();
   return true;
 }
