@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22041 $
- * $Author: csoutheren $
- * $Date: 2009-02-11 23:24:45 +0000 (Wed, 11 Feb 2009) $
+ * $Revision: 23300 $
+ * $Author: rjongbloed $
+ * $Date: 2009-08-28 04:43:32 -0500 (Fri, 28 Aug 2009) $
  */
 
 #ifdef __GNUC__
@@ -52,9 +52,6 @@ static const char DefaultYUVFileName[] = "*.yuv";
 
 #define new PNEW
 
-namespace PWLibStupidLinkerHacks {
-int loadVideoFileStuff;
-}; 
 
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoInputDevice_YUVFile
@@ -349,8 +346,6 @@ void PVideoInputDevice_YUVFile::FillRect(BYTE * frame,
                    int rectWidth, int rectHeight,
                    int r, int g,  int b)
 {
-// PTRACE(0,"x,y is"<<xPos<<" "<<yPos<<" and size is "<<rectWidth<<" "<<rectHeight);
-
   //This routine fills a region of the video image with data. It is used as the central
   //point because one only has to add other image formats here.
 
