@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23950 $
+ * $Revision: 24659 $
  * $Author: rjongbloed $
- * $Date: 2010-01-20 00:21:36 -0600 (Wed, 20 Jan 2010) $
+ * $Date: 2010-08-12 10:42:07 -0500 (Thu, 12 Aug 2010) $
  */
 
 #include <ptlib.h>
@@ -314,9 +314,9 @@ PString PTime::AsString(TimeFormat format, int zone) const
     case RFC1123 :
       return AsString("wwwe, dd MMME yyyy hh:mm:ss z", zone);
     case ShortISO8601 :
-      return AsString("yyyyMMddThhmmssZ");
+      return AsString("yyyyMMddThhmmssZ", zone);
     case LongISO8601 :
-      return AsString("yyyy-MM-dd T hh:mm:ss Z");
+      return AsString("yyyy-MM-dd T hh:mm:ss Z", zone);
     default :
       break;
   }
