@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23518 $
+ * $Revision: 24142 $
  * $Author: rjongbloed $
- * $Date: 2009-09-23 06:52:27 +0000 (Wed, 23 Sep 2009) $
+ * $Date: 2010-03-22 03:01:34 -0500 (Mon, 22 Mar 2010) $
  */
 
 #ifdef __GNUC__
@@ -678,23 +678,33 @@ void PSafePtrBase::ExitSafetyMode(ExitSafetyModeOption ref)
   if (ref == WithDereference && currentObject->SafeDereference()) {
     PSafeObject * objectToDelete = currentObject;
     currentObject = NULL;
-    PTRACE(6, "SafeColl\tDeleting object (" << objectToDelete << ')');
-    delete objectToDelete;
+    DeleteObject(objectToDelete);
   }
 }
 
 
+void PSafePtrBase::DeleteObject(PSafeObject * obj)
+{
+  if (obj == NULL)
+    return;
+
+  PTRACE(6, "SafeColl\tDeleting object (" << obj << ')');
+  delete obj;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 PSafePtrMultiThreaded::PSafePtrMultiThreaded(PSafeObject * obj, PSafetyMode mode)
   : PSafePtrBase(NULL, mode)
+  , m_objectToDelete(NULL)
 {
-  m_mutex.Wait();
+  Lock();
 
   currentObject = obj;
   EnterSafetyMode(WithReference);
 
-  m_mutex.Signal();
+  Unlock();
 }
 
 
@@ -702,13 +712,14 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafeCollection & safeCollect
                                              PSafetyMode mode,
                                              PINDEX idx)
   : PSafePtrBase(NULL, mode)
+  , m_objectToDelete(NULL)
 {
-  m_mutex.Wait();
+  Lock();
 
   collection = &safeCollection;
   Assign(idx);
 
-  m_mutex.Signal();
+  Unlock();
 }
 
 
@@ -716,20 +727,22 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafeCollection & safeCollect
                                              PSafetyMode mode,
                                              PSafeObject * obj)
   : PSafePtrBase(NULL, mode)
+  , m_objectToDelete(NULL)
 {
-  m_mutex.Wait();
+  Lock();
 
   collection = &safeCollection;
   Assign(obj);
 
-  m_mutex.Signal();
+  Unlock();
 }
 
 
 PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafePtrMultiThreaded & enumerator)
+  : m_objectToDelete(NULL)
 {
+  Lock();
   enumerator.m_mutex.Wait();
-  m_mutex.Wait();
 
   collection = enumerator.collection;
   currentObject = enumerator.currentObject;
@@ -737,17 +750,17 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafePtrMultiThreaded & enume
 
   EnterSafetyMode(WithReference);
 
-  m_mutex.Signal();
   enumerator.m_mutex.Signal();
+  Unlock();
 }
 
 
 PSafePtrMultiThreaded::~PSafePtrMultiThreaded()
 {
-  m_mutex.Wait();
+  Lock();
   ExitSafetyMode(WithDereference);
   currentObject = NULL;
-  m_mutex.Signal();
+  Unlock();
 }
 
 
@@ -760,9 +773,9 @@ PObject::Comparison PSafePtrMultiThreaded::Compare(const PObject & obj) const
 
 void PSafePtrMultiThreaded::SetNULL()
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::SetNULL();
-  m_mutex.Signal();
+  Unlock();
 }
 
 
@@ -775,59 +788,74 @@ PBoolean PSafePtrMultiThreaded::SetSafetyMode(PSafetyMode mode)
 
 void PSafePtrMultiThreaded::Assign(const PSafePtrMultiThreaded & ptr)
 {
+  Lock();
   ptr.m_mutex.Wait();
-  m_mutex.Wait();
   PSafePtrBase::Assign(ptr);
-  m_mutex.Signal();
   ptr.m_mutex.Signal();
+  Unlock();
 }
 
 
 void PSafePtrMultiThreaded::Assign(const PSafePtrBase & ptr)
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::Assign(ptr);
-  m_mutex.Signal();
+  Unlock();
 }
 
 
 void PSafePtrMultiThreaded::Assign(const PSafeCollection & safeCollection)
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::Assign(safeCollection);
-  m_mutex.Signal();
+  Unlock();
 }
 
 
 void PSafePtrMultiThreaded::Assign(PSafeObject * obj)
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::Assign(obj);
-  m_mutex.Signal();
+  Unlock();
 }
 
 
 void PSafePtrMultiThreaded::Assign(PINDEX idx)
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::Assign(idx);
-  m_mutex.Signal();
+  Unlock();
 }
 
 
 void PSafePtrMultiThreaded::Next()
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::Next();
-  m_mutex.Signal();
+  Unlock();
 }
 
 
 void PSafePtrMultiThreaded::Previous()
 {
-  m_mutex.Wait();
+  Lock();
   PSafePtrBase::Previous();
+  Unlock();
+}
+
+
+void PSafePtrMultiThreaded::DeleteObject(PSafeObject * obj)
+{
+  m_objectToDelete = obj;
+}
+
+
+void PSafePtrMultiThreaded::Unlock()
+{
+  PSafeObject * obj = m_objectToDelete;
+  m_objectToDelete = NULL;
   m_mutex.Signal();
+  PSafePtrBase::DeleteObject(obj);
 }
 
 
