@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 20498 $
- * $Author: shorne $
- * $Date: 2008-06-26 03:05:26 +0200 (jeu 26 jun 2008) $
+ * $Revision: 23962 $
+ * $Author: rjongbloed $
+ * $Date: 2010-01-21 22:39:06 -0600 (Thu, 21 Jan 2010) $
  */
 
 #ifdef __GNUC__
@@ -79,20 +79,19 @@ ostream & operator<<(ostream & strm, PVideoDevice::VideoFormat fmt)
 
 
 //Colour format bit per pixel table.
-// These are in rough order of colour gamut size
+// These are in rough order of colour gamut size and "popularity"
 static struct {
   const char * colourFormat;
   unsigned     bitsPerPixel;
 } colourFormatBPPTab[] = {
-  { "YUV420",  12 },
   { "YUV420P", 12 },
   { "I420",    12 },
+  { "IYUV",    12 },
+  { "YUV420",  12 },
   { "RGB32",   32 },
   { "BGR32",   32 },
   { "RGB24",   24 },
   { "BGR24",   24 },
-  { "MJPEG",   16 },
-  { "JPEG",    16 },
   { "YUY2",    16 },
   { "YUV422",  16 },
   { "YUV422P", 16 },
@@ -100,7 +99,6 @@ static struct {
   { "YUV411P", 12 },
   { "RGB565",  16 },
   { "RGB555",  16 },
-  { "IYUV",    12 },
   { "YUV410",  10 },
   { "YUV410P", 10 },
   { "Grey",     8 },
@@ -116,34 +114,46 @@ static VideoDevice * CreateDeviceWithDefaults(PString & adjustedDeviceName,
                                               const PString & driverName,
                                               PPluginManager * pluginMgr)
 {
-  PString adjustedDriverName = driverName;
+  if (adjustedDeviceName == "*")
+    adjustedDeviceName.MakeEmpty();
 
-  if (adjustedDeviceName.IsEmpty() || adjustedDeviceName == "*") {
-    if (driverName.IsEmpty() || driverName == "*") {
+  PString adjustedDriverName = driverName;
+  if (adjustedDriverName == "*")
+    adjustedDriverName.MakeEmpty();
+
+  if (adjustedDeviceName.IsEmpty()) {
+    if (adjustedDriverName.IsEmpty()) {
       PStringArray drivers = VideoDevice::GetDriverNames(pluginMgr);
       if (drivers.IsEmpty())
         return NULL;
 
-      // Give precedence to drivers like camera grabbers, leave out the fail safe types such as NULL
-      PINDEX driverIndex;
-      for (driverIndex = drivers.GetSize()-1; driverIndex > 0; --driverIndex) {
-        static const char * lowPriorityDrivers[] = {
-          "YUVFile", "FakeVideo", "NULLOutput"
-        };
-        PINDEX i;
-        for (i = 0; i < PARRAYSIZE(lowPriorityDrivers); i++) {
-          if (drivers[driverIndex] == lowPriorityDrivers[i])
+      // Give precedence to drivers like camera grabbers, Window
+      static const char * prioritisedDrivers[] = {
+        "Window", "SDL", "DirectShow", "VideoForWindows", "V4L", "V4L2", "1394DC", "1394AVC", "BSDCAPTURE", "FakeVideo", "NULLOutput"
+      };
+      for (PINDEX i = 0; i < PARRAYSIZE(prioritisedDrivers); i++) {
+        PINDEX driverIndex = drivers.GetValuesIndex(PString(prioritisedDrivers[i]));
+        if (driverIndex != P_MAX_INDEX) {
+          PStringArray devices = VideoDevice::GetDriversDeviceNames(drivers[driverIndex]);
+          if (!devices.IsEmpty()) {
+            adjustedDeviceName = devices[0];
+            adjustedDriverName = drivers[driverIndex];
             break;
+          }
         }
-        if (i == PARRAYSIZE(lowPriorityDrivers))
-          break;
       }
-      adjustedDriverName = drivers[driverIndex];
+
+      if (adjustedDriverName.IsEmpty())
+        adjustedDriverName = drivers[0];
     }
 
-    PStringArray devices = VideoDevice::GetDriversDeviceNames(adjustedDriverName);
-    if (!devices.IsEmpty())
+    if (adjustedDeviceName.IsEmpty()) {
+      PStringArray devices = VideoDevice::GetDriversDeviceNames(adjustedDriverName);
+      if (devices.IsEmpty())
+        return NULL;
+
       adjustedDeviceName = devices[0];
+    }
   }
 
   return VideoDevice::CreateDeviceByName(adjustedDeviceName, adjustedDriverName, pluginMgr);
@@ -163,6 +173,20 @@ PVideoFrameInfo::PVideoFrameInfo()
 }
 
 
+PVideoFrameInfo::PVideoFrameInfo(unsigned        width,
+                                 unsigned        height,
+                                 const PString & format,
+                                 unsigned        rate,
+                                 ResizeMode      resize)
+  : frameWidth(width)
+  , frameHeight(height)
+  , frameRate(rate)
+  , colourFormat(format)
+  , resizeMode(resize)
+{
+}
+
+
 PBoolean PVideoFrameInfo::SetFrameSize(unsigned width, unsigned height)
 {
   if (width < 8 || height < 8)
@@ -246,50 +270,108 @@ PINDEX PVideoFrameInfo::CalculateFrameBytes(unsigned width, unsigned height,
 }
  
 
-PBoolean PVideoFrameInfo::ParseSize(const PString & str, unsigned & width, unsigned & height)
+bool PVideoFrameInfo::Parse(const PString & str)
 {
-  static struct {
-    const char * name;
-    unsigned width;
-    unsigned height;
-  } const sizeTable[] = {
-      { "cif",    PVideoDevice::CIFWidth,   PVideoDevice::CIFHeight   },
-      { "qcif",   PVideoDevice::QCIFWidth,  PVideoDevice::QCIFHeight  },
-      { "sqcif",  PVideoDevice::SQCIFWidth, PVideoDevice::SQCIFHeight },
-      { "cif4",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
-      { "4cif",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
-      { "cif16",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
-      { "16cif",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
+  PString newFormat = colourFormat;
+  PINDEX formatOffset = str.Find(':');
+  if (formatOffset == 0)
+    return false;
 
-      { "ccir601",720,                      486                       },
-      { "ntsc",   720,                      480                       },
-      { "pal",    768,                      576                       },
-      { "hdtvp",  1280,                     720                       },
-      { "hd720",  1280,                     720                       },
-      { "hdtvi",  1920,                     1080                      },
-      { "hd1080", 1920,                     1080                      },
+  if (formatOffset == P_MAX_INDEX)
+    formatOffset = 0;
+  else
+    newFormat = str.Left(formatOffset++);
 
-      { "cga",    320,                      240                       },
-      { "vga",    640,                      480                       },
-      { "wvga",   854,                      480                       },
-      { "svga",   800,                      600                       },
-      { "xga",    1024,                     768                       },
-      { "sxga",   1280,                     1024                      },
-      { "wsxga",  1440,                     900                       },
-      { "sxga+",  1400,                     1050                      },
-      { "wsxga+", 1680,                     1050                      },
-      { "uxga",   1600,                     1200                      },
-      { "wuxga",  1920,                     1200                      },
-      { "qxga",   2048,                     1536                      },
-      { "wqxga",  2560,                     1600                      },
-      { }
-  };
 
-  for (int i = 0; sizeTable[i].name != NULL; i++) {
-    if (str *= sizeTable[i].name) {
-      width = sizeTable[i].width;
-      height = sizeTable[i].height;
-      return PTrue;
+  ResizeMode newMode = resizeMode;
+  PINDEX resizeOffset = str.Find('/', formatOffset);
+  if (resizeOffset != P_MAX_INDEX) {
+    static struct {
+      const char * name;
+      ResizeMode   mode;
+    } const ResizeNames[] = {
+      { "scale",   eScale },
+      { "resize",  eScale },
+      { "centre",  eCropCentre },
+      { "center",  eCropCentre },
+      { "crop",    eCropTopLeft },
+      { "topleft", eCropTopLeft }
+    };
+
+    PCaselessString crop = str.Mid(resizeOffset+1);
+    PINDEX resizeIndex = 0;
+    while (crop != ResizeNames[resizeIndex].name) {
+      if (++resizeIndex >= PARRAYSIZE(ResizeNames))
+        return false;
+    }
+    newMode = ResizeNames[resizeIndex].mode;
+  }
+
+
+  int newRate = frameRate;
+  PINDEX rateOffset = str.Find('@', formatOffset);
+  if (rateOffset == P_MAX_INDEX)
+    rateOffset = resizeOffset;
+  else {
+    newRate = str.Mid(rateOffset+1).AsInteger();
+    if (newRate < 1 || newRate > 100)
+      return false;
+  }
+
+  if (!ParseSize(str(formatOffset, rateOffset-1), frameWidth, frameHeight))
+    return false;
+
+  colourFormat = newFormat;
+  frameRate = newRate;
+  resizeMode = newMode;
+  return true;
+}
+
+
+static struct {
+  const char * name;
+  unsigned width;
+  unsigned height;
+} const SizeTable[] = {
+    { "CIF",    PVideoDevice::CIFWidth,   PVideoDevice::CIFHeight   },
+    { "QCIF",   PVideoDevice::QCIFWidth,  PVideoDevice::QCIFHeight  },
+    { "SQCIF",  PVideoDevice::SQCIFWidth, PVideoDevice::SQCIFHeight },
+    { "CIF4",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
+    { "4CIF",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
+    { "CIF16",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
+    { "16CIF",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
+
+    { "CCIR601",720,                      486                       },
+    { "NTSC",   720,                      480                       },
+    { "PAL",    768,                      576                       },
+    { "HDTVP",  1280,                     720                       },
+    { "HD720",  1280,                     720                       },
+    { "HDTVI",  1920,                     1080                      },
+    { "HD1080", 1920,                     1080                      },
+
+    { "CGA",    320,                      240                       },
+    { "VGA",    640,                      480                       },
+    { "WVGA",   854,                      480                       },
+    { "SVGA",   800,                      600                       },
+    { "XGA",    1024,                     768                       },
+    { "SXGA",   1280,                     1024                      },
+    { "WSXGA",  1440,                     900                       },
+    { "SXGA+",  1400,                     1050                      },
+    { "WSXGA+", 1680,                     1050                      },
+    { "UXGA",   1600,                     1200                      },
+    { "WUXGA",  1920,                     1200                      },
+    { "QXGA",   2048,                     1536                      },
+    { "WQXGA",  2560,                     1600                      },
+    { }
+};
+
+bool PVideoFrameInfo::ParseSize(const PString & str, unsigned & width, unsigned & height)
+{
+  for (int i = 0; SizeTable[i].name != NULL; i++) {
+    if (str *= SizeTable[i].name) {
+      width = SizeTable[i].width;
+      height = SizeTable[i].height;
+      return true;
     }
   }
 
@@ -297,6 +379,17 @@ PBoolean PVideoFrameInfo::ParseSize(const PString & str, unsigned & width, unsig
 }
 
 
+PString PVideoFrameInfo::AsString(unsigned width, unsigned height)
+{
+  for (int i = 0; SizeTable[i].name != NULL; i++) {
+    if (SizeTable[i].width == width && SizeTable[i].height == height)
+      return SizeTable[i].name;
+  }
+
+  return psprintf("%ux%u", width, height);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoDevice
 
@@ -542,6 +635,24 @@ PBoolean PVideoDevice::SetColourFormatConverter(const PString & newColourFmt)
     PTRACE(3, "PVidDev\tSetColourFormatConverter success for native " << colourFmt);    
     return PTrue;
   }
+
+  // check to see if we have detected this device before, and try the same combination worked last time
+  if (CanCaptureVideo()) {
+    PString previousColourFormat       = "BGR24";
+
+    PTRACE(4,"PVidDev\tSetColourFormatConverter trying, want " << colourFmt << " trying " << previousColourFormat);
+    if (SetColourFormat(previousColourFormat)) {
+      src.SetColourFormat(previousColourFormat);
+      dst.SetColourFormat(colourFmt);
+      converter = PColourConverter::Create(src, dst);
+      if (converter != NULL) {
+        PTRACE(3, "PVidDev\tSetColourFormatConverter succeeded for " << colourFmt << " and device using " << previousColourFormat);
+        converter->SetVFlipState(nativeVerticalFlip);
+        return PTrue;
+      }
+    }
+    PTRACE(4,"PVidDev\tSetColourFormatConverter " << previousColourFormat << " did not work");
+  }
   
   /************************
     Eventually, need something more sophisticated than this, but for the
@@ -897,6 +1008,25 @@ PString PVideoControlInfo::AsString(const InputControlType & ctype)
 	return PString();
 }
 
+////////////////////////////////////////////////////////////////////////////////////////////
+
+PString PVideoInteractionInfo::AsString(const InputInteractType & ctype)
+{
+	switch (ctype) {
+		case InteractKey:
+			return "Remote Key Press";
+		case InteractMouse:
+			return "Remote Mouse Move/Click";
+		case InteractNavigate:
+			return "Remote Navigation";
+		case InteractRTSP:
+			return "Remote RTSP Commands";
+		case InteractOther:
+			return "Custom/Other";
+	}
+	return PString();
+}
+
 /////////////////////////////////////////////////////////////////////////////////////////////
 
 PVideoInputControl::~PVideoInputControl()  
@@ -1031,6 +1161,12 @@ PBoolean PVideoOutputDevice::GetPosition(int &, int &) const
 }
 
 
+bool PVideoOutputDevice::SetPosition(int, int)
+{
+  return false;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDeviceRGB
 
@@ -1083,6 +1219,9 @@ PBoolean PVideoOutputDeviceRGB::SetFrameSize(unsigned width, unsigned height)
 {
   PWaitAndSignal m(mutex);
 
+  if (frameWidth == width && frameHeight == height)
+    return true;
+
   if (!PVideoOutputDevice::SetFrameSize(width, height))
     return PFalse;
 
