@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23236 $
+ * $Revision: 24468 $
  * $Author: rjongbloed $
- * $Date: 2009-08-20 19:14:22 -0500 (Thu, 20 Aug 2009) $
+ * $Date: 2010-06-07 01:47:45 -0500 (Mon, 07 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -54,8 +54,6 @@
   #endif
 #endif
 
-#include "../common/pglobalstatic.cxx"
-
 
 #define new PNEW
 
@@ -65,22 +63,39 @@
 
 PTime::PTime()
 {
-  // Magic constant to convert epoch from 1601 to 1970
-  static const PInt64 delta = ((PInt64)369*365+(369/4)-3)*24*60*60U;
-  static const PInt64 scale = 10000000;
-
-  PInt64 timestamp;
+  FILETIME timestamp;
 
 #ifndef _WIN32_WCE
-  GetSystemTimeAsFileTime((LPFILETIME)&timestamp);
+  GetSystemTimeAsFileTime(&timestamp);
 #else
   SYSTEMTIME SystemTime;
   GetSystemTime(&SystemTime);
-  SystemTimeToFileTime(&SystemTime, (LPFILETIME)&timestamp);
+  SystemTimeToFileTime(&SystemTime, &timestamp);
 #endif
 
-  theTime = (time_t)(timestamp/scale - delta);
-  microseconds = (long)(timestamp%scale/10);
+  SetFromFileTime(timestamp);
+}
+
+
+PTime::PTime(const FILETIME & timestamp)
+{
+  SetFromFileTime(timestamp);
+}
+
+
+void PTime::SetFromFileTime(const FILETIME & timestamp)
+{
+  // Magic constant to convert epoch from 1601 to 1970
+  static const ULONGLONG delta = ((PInt64)369*365+(369/4)-3)*24*60*60U;
+  static const ULONGLONG scale = 10000000;
+
+  ULARGE_INTEGER i;
+  i.HighPart = timestamp.dwHighDateTime;
+  i.LowPart = timestamp.dwLowDateTime;
+  i.QuadPart;
+
+  theTime = (time_t)(i.QuadPart/scale - delta);
+  microseconds = (long)(i.QuadPart%scale/10);
 }
 
 #ifdef UNICODE
@@ -233,14 +248,14 @@ unsigned PTimer::Resolution()
     return 1;
 
 #ifndef _WIN32_WCE
-  DWORD err = GetLastError();
+  DWORD err = ::GetLastError();
   DWORD timeAdjustment;
   DWORD timeIncrement;
   BOOL timeAdjustmentDisabled;
   if (GetSystemTimeAdjustment(&timeAdjustment, &timeIncrement, &timeAdjustmentDisabled))
     return timeIncrement/10000;
 
-  err = GetLastError();
+  err = ::GetLastError();
 #endif
 
   return 55;
@@ -432,7 +447,7 @@ PBoolean PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clust
 
     PString drive = "A:";
     while (WNetAddConnection(root, NULL, drive) != NO_ERROR) {
-      if (GetLastError() != ERROR_ALREADY_ASSIGNED)
+      if (::GetLastError() != ERROR_ALREADY_ASSIGNED)
         return PFalse;
       drive[0]++;
     }
@@ -574,7 +589,7 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
   if (status != -2)
     osError = errno;
   else {
-    osError = GetLastError();
+    osError = ::GetLastError();
     switch (osError) {
       case ERROR_INVALID_HANDLE :
       case WSAEBADF :
@@ -693,9 +708,9 @@ UINT __stdcall PThread::MainFunction(void * threadPtr)
 #endif
 */
 
-  process.activeThreadMutex.Wait();
-  process.activeThreads.SetAt(thread->threadId, thread);
-  process.activeThreadMutex.Signal();
+  process.m_activeThreadMutex.Wait();
+  process.m_activeThreads[thread->threadId] = thread;
+  process.m_activeThreadMutex.Signal();
 
   process.SignalTimerChange();
 
@@ -731,6 +746,7 @@ void PThread::Win32AttachThreadInput()
 
 PThread::PThread()
   : autoDelete(false)
+  , m_isProcess(true)
   , originalStackSize(0)
   , threadHandle(GetCurrentThread())
   , threadId(GetCurrentThreadId())
@@ -738,14 +754,15 @@ PThread::PThread()
   if (!PProcess::IsInitialised())
     return;
 
+  m_isProcess = false;
   autoDelete = true;
   DuplicateHandle(GetCurrentProcess(), threadHandle, GetCurrentProcess(), &threadHandle, 0, 0, DUPLICATE_SAME_ACCESS);
 
   PProcess & process = PProcess::Current();
 
-  process.activeThreadMutex.Wait();
-  process.activeThreads.SetAt(threadId, this);
-  process.activeThreadMutex.Signal();
+  process.m_activeThreadMutex.Wait();
+  process.m_activeThreads[threadId] = this;
+  process.m_activeThreadMutex.Signal();
 
   process.deleteThreadMutex.Wait();
   process.autoDeleteThreads.Append(this);
@@ -758,6 +775,7 @@ PThread::PThread(PINDEX stackSize,
                  Priority priorityLevel,
                  const PString & name)
   : threadName(name)
+  , m_isProcess(false)
 {
   PAssert(stackSize > 0, PInvalidParameter);
   originalStackSize = stackSize;
@@ -788,107 +806,60 @@ PThread::PThread(PINDEX stackSize,
 
 PThread::~PThread()
 {
-  if (originalStackSize <= 0)
+  if (m_isProcess)
     return;
 
   CleanUp();
 }
 
 
-#if PTRACING
-static ULONGLONG ConvertTo64(const FILETIME & ft)
+static ULONGLONG GetMillisecondFromFileTime(const FILETIME & ft)
 {
   ULARGE_INTEGER i;
   i.HighPart = ft.dwHighDateTime;
   i.LowPart = ft.dwLowDateTime;
-  return i.QuadPart;
+  return (i.QuadPart+9999)/10000;
 }
 
-static void OutputTime(ostream & str, ULONGLONG time)
+
+bool PThread::GetTimes(Times & times)
 {
-  ULONGLONG upperbound = 10000;
-  ULONGLONG scale1 = 10;
-  ULONGLONG scale2 = 1;
-  int units = 0;
-  int decimals = 1;
+  FILETIME created, exit, kernel, user;
+  exit.dwHighDateTime = exit.dwLowDateTime = 0;
+  if (!GetThreadTimes(GetHandle(), &created, &exit, &kernel, &user))
+    return false;
 
-  while (time >= upperbound) {
-    if (--decimals == 0) {
-      if (units == 2) {
-        str << time/scale1 << 's';
-        return;
-      }
+  times.m_kernel.SetInterval(GetMillisecondFromFileTime(kernel));
+  times.m_user.SetInterval(GetMillisecondFromFileTime(user));
+  if (exit.dwHighDateTime == 0 && exit.dwLowDateTime == 0)
+    times.m_real = PTime() - PTime(created);
+  else
+    times.m_real = PTime(exit) - PTime(created);
 
-      scale1 *= 1000;
-      decimals = 3;
-      ++units;
-    }
-    upperbound *= 10;
-    scale2 *= 10;
-  }
-
-  static const char * const UnitNames[] = { "us", "ms", "s" };
-  str << time/scale1
-      << '.' << setfill('0') << setw(decimals)
-      << (time%scale1)/scale2
-      << UnitNames[units];
+  return true;
 }
 
-static void OutputThreadTimes(const PThread & thread)
-{
-  if (!PTrace::CanTrace(3))
-    return;
-
-  FILETIME created_f, exit_f, kernel_f, user_f;
-  exit_f.dwHighDateTime = exit_f.dwLowDateTime = 0;
-  if (!GetThreadTimes(thread.GetHandle(), &created_f, &exit_f, &kernel_f, &user_f))
-    return;
-
-  ULONGLONG created = ConvertTo64(created_f);
-  ULONGLONG exit    = ConvertTo64(exit_f);
-  ULONGLONG kernel  = ConvertTo64(kernel_f);
-  ULONGLONG user    = ConvertTo64(user_f);
-
-  ostream & output = PTrace::Begin(3, __FILE__, __LINE__);
-  output << "PTLib\tThread ended: name=\"" << thread.GetThreadName() << "\", kernel=";
-  OutputTime(output, kernel);
-  output << ", user=";
-  OutputTime(output, user);
-  output << ", elapsed=";
-  OutputTime(output, exit-created);
-  if (exit > created) {
-    ULONGLONG percent = 1000*(kernel+user)/(exit-created);
-    if (percent > 0)
-      output << " (" << percent/10 << '.' << percent%10 << "%)";
-    else
-      output << " (<0.1%)";
-  }
-  output << PTrace::End;
-}
-#else
-#define OutputThreadTimes(t)
-#endif
 
 void PThread::CleanUp()
 {
   PProcess & process = PProcess::Current();
-  process.activeThreadMutex.Wait();
-  process.activeThreads.SetAt(threadId, NULL);
-  process.activeThreadMutex.Signal();
+  process.m_activeThreadMutex.Wait();
+  process.m_activeThreads.erase(threadId);
+  process.m_activeThreadMutex.Signal();
 
   if (!IsTerminated())
     Terminate();
 
-  if (threadHandle != NULL) {
-    OutputThreadTimes(*this);
+  if (threadHandle != NULL)
     CloseHandle(threadHandle);
-  }
 }
 
 
 void PThread::Restart()
 {
-  PAssert(IsTerminated(), "Cannot restart running thread");
+  if (!PAssert(originalStackSize != 0, "Cannot restart process/external thread") ||
+      !PAssert(IsTerminated(), "Cannot restart running thread"))
+    return;
 
   CleanUp();
 
@@ -977,7 +948,7 @@ void PThread::Resume()
 
 PBoolean PThread::IsSuspended() const
 {
-  if (threadHandle == PThread::Current()->GetHandle())
+  if (this == PThread::Current())
     return false;
 
   SuspendThread(threadHandle);
@@ -1140,14 +1111,20 @@ void PProcess::HouseKeepingThread::Main()
 }
 
 
-void PProcess::SignalTimerChange()
+bool PProcess::SignalTimerChange()
 {
+  if (m_shuttingDown) 
+    return false;
+
   deleteThreadMutex.Wait();
+
   if (houseKeeper == NULL)
     houseKeeper = new HouseKeepingThread;
   else
     houseKeeper->m_breakBlock.Signal();
+
   deleteThreadMutex.Signal();
+  return true;
 }
 
 
@@ -1163,15 +1140,16 @@ PProcess::~PProcess()
 
   // Get rid of the house keeper (majordomocide)
   delete houseKeeper;
+  houseKeeper = NULL;
 
   // OK, if there are any left we get really insistent...
-  activeThreadMutex.Wait();
-  for (PINDEX i = 0; i < activeThreads.GetSize(); i++) {
-    PThread & thread = activeThreads.GetDataAt(i);
+  m_activeThreadMutex.Wait();
+  for (ThreadMap::iterator it = m_activeThreads.begin(); it != m_activeThreads.end(); ++it) {
+    PThread & thread = *it->second;
     if (this != &thread && !thread.IsTerminated())
       TerminateThread(thread.GetHandle(), 1);  // With extreme prejudice
   }
-  activeThreadMutex.Signal();
+  m_activeThreadMutex.Signal();
 
   deleteThreadMutex.Wait();
   autoDeleteThreads.RemoveAll();
@@ -1468,7 +1446,7 @@ PBoolean PSemaphore::Wait(const PTimeInterval & timeout)
 void PSemaphore::Signal()
 {
   if (!ReleaseSemaphore(handle, 1, NULL))
-    PAssertOS(GetLastError() != ERROR_INVALID_HANDLE);
+    PAssertOS(::GetLastError() != ERROR_INVALID_HANDLE);
   SetLastError(ERROR_SUCCESS);
 }
 
@@ -1527,7 +1505,7 @@ void PSyncPoint::Signal()
 
 PDynaLink::PDynaLink()
 {
-  _hDLL = NULL;
+  m_hDLL = NULL;
 }
 
 
@@ -1551,79 +1529,91 @@ PString PDynaLink::GetExtension()
 
 PBoolean PDynaLink::Open(const PString & name)
 {
+  m_lastError.MakeEmpty();
+
   PVarString filename = name;
-  _hDLL = LoadLibrary(filename);
-  return _hDLL != NULL;
+#ifndef _WIN32_WCE
+  m_hDLL = LoadLibraryEx(filename, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
+#else
+  m_hDLL = LoadLibrary(filename);
+#endif
+  if (m_hDLL != NULL)
+    return true;
+
+  m_lastError.sprintf("0x%x", ::GetLastError());
+  PTRACE(1, "DLL\tError loading DLL: " << m_lastError);
+  return false;
 }
 
 
 void PDynaLink::Close()
 {
-  if (_hDLL != NULL) {
-    FreeLibrary(_hDLL);
-    _hDLL = NULL;
+  if (m_hDLL != NULL) {
+    FreeLibrary(m_hDLL);
+    m_hDLL = NULL;
   }
 }
 
 
 PBoolean PDynaLink::IsLoaded() const
 {
-  return _hDLL != NULL;
+  return m_hDLL != NULL;
 }
 
 
 PString PDynaLink::GetName(PBoolean full) const
 {
   PFilePathString str;
-  if (_hDLL != NULL) 
-{
-
+  if (m_hDLL != NULL) {
 #ifdef UNICODE
-  TCHAR path[_MAX_PATH];
-    GetModuleFileName(_hDLL, path, _MAX_PATH-1);
-    str=PString(path);
+    TCHAR path[_MAX_PATH];
+    GetModuleFileName(m_hDLL, path, _MAX_PATH-1);
+    str = PString(path);
 #else
-    GetModuleFileName(_hDLL, str.GetPointer(_MAX_PATH), _MAX_PATH-1);
+    GetModuleFileName(m_hDLL, str.GetPointer(_MAX_PATH), _MAX_PATH-1);
 #endif
-    if (!full) 
-    {
+    if (!full) {
       str.Delete(0, str.FindLast('\\')+1);
       PINDEX pos = str.Find(".DLL");
       if (pos != P_MAX_INDEX)
         str.Delete(pos, P_MAX_INDEX);
     }
+    str.MakeMinimumSize();
   }
-  str.MakeMinimumSize();
   return str;
 }
 
 
 PBoolean PDynaLink::GetFunction(PINDEX index, Function & func)
 {
-  if (_hDLL == NULL)
-    return PFalse;
+  m_lastError.MakeEmpty();
 
-  FARPROC p = GetProcAddress(_hDLL, (LPTSTR)(DWORD)LOWORD(index));
-  if (p == NULL)
-    return PFalse;
+  if (m_hDLL == NULL)
+    return false;
 
-  func = (Function)p;
-  return PTrue;
+  func = (Function)GetProcAddress(m_hDLL, (LPTSTR)(DWORD)LOWORD(index));
+  if (func != NULL)
+    return true;
+
+  m_lastError.sprintf("0x%x", ::GetLastError());
+  return false;
 }
 
 
 PBoolean PDynaLink::GetFunction(const PString & name, Function & func)
 {
-  if (_hDLL == NULL)
+  m_lastError.MakeEmpty();
+
+  if (m_hDLL == NULL)
     return PFalse;
 
   PVarString funcname = name;
-  FARPROC p = GetProcAddress(_hDLL, funcname);
-  if (p == NULL)
-    return PFalse;
+  func = (Function)GetProcAddress(m_hDLL, funcname);
+  if (func != NULL)
+    return true;
 
-  func = (Function)p;
-  return PTrue;
+  m_lastError.sprintf("0x%x", ::GetLastError());
+  return false;
 }
 
 
